/*****************************************************************************
*                                                                            *
*       Name            : SPI.H                                              *
*       Autor           : A.Haeusler                                         *
*       Funktion        : FRAM SPI Routinen Mode 3							 *
*                         											 		 *
*       Prozessor       : H8S-2378                                           *
*                                                                            *
*       Datum           : 30.01.2006                                         *
*                                                                            *
*       Spezielles      :                                                    *
*                                                                            *
*****************************************************************************/

#ifndef _FM25XX_INC
#define _FM25XX_INC

/****************** DEFINITION DER VERWENDETEN MAKROS ***********************/

#define MEMORY_WRITE_TIMING		FAST		// FAST

/******************** DEFINITION DER PORTS FUER SPI *************************/    
/*
 	
	Peripherie		Port		CPU			Kommentar
	
		/CS1 		= P10		/CS1		Chip Select Nr.1
		/CS2 		= P11		/CS2		Chip Select Nr.2
		/CS3 		= P12		/CS3		Chip Select Nr.3
		/CS4 		= P13		/CS4		Chip Select Nr.4
 		SO 			= P16		SI			Serieller Daten Eingang (CPU)
 		SI 			= P15		SO  		Serieller Daten Ausgang (CPU)
		/WP			= P17		/WP			Schreibschutz Leitung
		SCK			= P14		SCK			Clock Leitung

	P1DDR= 0xFD;	// Bit=0 > Input / Bit=1 > Output
								 
					// Bit 7 / P17> Output: FRAM_WP
					// Bit 6 / P16> Input : FRAM_SO
					// Bit 5 / P15> Output: FRAM_SI
					// Bit 4 / P14> Output: FRAM_SCK

					// Bit 3 / P13> Output: FRAM_CS4
					// Bit 2 / P12> Output: FRAM_CS3
					// Bit 1 / P11> Output: FRAM_CS2
					// Bit 0 / P10> Output: FRAM_CS1


// DDR Output = 1 / DDR Input = 0


// Setzt Chip Select auf Ausgang 
#define CS1_OUT      p1ddr_tmp &= ~0x01; P1DDR = p1ddr_tmp;

// Setzt Chip Select auf Ausgang 
#define CS2_OUT      p1ddr_tmp &= ~0x02; P1DDR = p1ddr_tmp;

// Setzt Chip Select auf Ausgang 
#define CS3_OUT      p1ddr_tmp &= ~0x04; P1DDR = p1ddr_tmp;

// Setzt Chip Select auf Ausgang 
#define CS4_OUT      p1ddr_tmp &= ~0x08; P1DDR = p1ddr_tmp;

// Setzt Clockleitung auf Ausgang       	   
#define SCK_OUT     p1ddr_tmp |= 0x10; P1DDR = p1ddr_tmp;  

// Setzt Dataleitung auf Ausgang
#define SO_OUT		p1ddr_tmp |= 0x20; P1DDR = p1ddr_tmp; 

// Setzt Dataleitung auf Ausgang
#define SI_IN		p1ddr_tmp |= 0x40; P1DDR = p1ddr_tmp; 

// Setzt Write Protect Leitung auf Ausgang       	   
#define WP_OUT     p1ddr_tmp |= 0x80; P1DDR = p1ddr_tmp;

*/

#define SET_SPI_PORT_DIR 	P1DDR=0xFD;         	 // Setzt der Portpins

#define CS1_OFF    			P1DR |= 0x01         	 // Setzt CS 1 aktiv
#define CS1_ON    			P1DR &= ~0x01         	 // Setzt CS 1 inaktiv

#define CS2_OFF    			P1DR |= 0x02         	 // Setzt CS 2 aktiv
#define CS2_ON    			P1DR &= ~0x02         	 // Setzt CS 2 inaktiv

#define CS3_OFF    			P1DR |= 0x04         	 // Setzt CS 3 aktiv
#define CS3_ON    			P1DR &= ~0x04         	 // Setzt CS 3 inaktiv

#define CS4_OFF    			P1DR |= 0x08         	 // Setzt CS 4 aktiv
#define CS4_ON    			P1DR &= ~0x08         	 // Setzt CS 4 inaktiv

#define SCK_1     			P1DR |= 0x10         	 // Setzt Clockleitung High
#define SCK_0     			P1DR &= ~0x10         	 // Setzt Clockleitung Low 
       	  
#define SO_1				P1DR |= 0x20         	 // Setzt Dataleitung High
#define SO_0      			P1DR &= ~0x20         	 // Setzt Dataleitung Low  

#define SI_IN_1   			(PORT1 & 0x40) == 0x40   // Liest Dataleitung=High 
#define SI_IN_0   			(PORT1 & 0x40) == 0x00   // Liest Dataleitung=Low   
                                                                    			
#define SPI_EEPROM_WP_ON	P1DR |= 0x80      	   	 // Schreibschutz ON	
#define SPI_EEPROM_WP_OFF 	P1DR &= ~0x80;  		 // Schreibschutz OFF 

#define Debug_SPI			0		// zum auffinden von Fehlern (1 = Meldungen ON)
//#define Debug_SPI			1

#define SPI_Mode			0		// 1 = Hardware SPI | 0 = Software SPI
//#define ISP_Mode			1

/************************* GLOBALE DEFINITIONEN *****************************/

#define SLOW   					0 				// Fr I2C Memory Write Timing  
#define FAST   					1 				
                                            	
/******************* DEFINITION EEPROM HARDWARE ADRESSE *********************/

// SPI COMMANDS --------------------------------------------------------------

#define CMD_WREN		0x06		// Set Write Enable Latch
#define CMD_WRDI		0x04		// Write Disable
#define CMD_RDSR		0x05		// Read Status Register
#define CMD_WRSR		0x01		// Write Status Register
#define CMD_READ		0x03		// Read Memory Data
#define CMD_WRITE		0x02		// Write Memory Data
		                                             	                                              	
/************************ TIMING DEFINITIONEN *******************************/ 

// H8S-2378
// Operating frequency (MHz) / Supply voltage (V) 
//        33MHz              /   3.0V to 3.6V 
// Minimum instruction execution time: 30ns
// 1 NOP(); @ 14,7456MHz = 68ns

#define SPI_TCSU        NOP()			// Chip Select Setup 10ns
#define SPI_TCSH        NOP()			// Chip Select Hold 10ns
#define SPI_TSU         NOP()			// Data Setup Time 5ns
#define SPI_TH        	NOP()			// Data Hold Time 5ns
#define SPI_TD        	NOP();NOP()		// Deselect Time 60ns
#define SPI_TCH        	NOP()			// Clock High Time 22ns
#define SPI_TCL        	NOP()			// Clock Low Time 22ns

#define SPI_WRITE_TIME  NOP();NOP()    	//
                                            	
/************************ SONSTIGE DEFINITIONEN *****************************/ 

                                             	                                           
/*************************** EXTERNE FUNKTIONEN *****************************/                                               

extern void delay( unsigned short Zeit );                                       
                                             
/******************************* PROTOTYPEN *********************************/  
    