/****************************************************************************/
/*##########################################################################*/
/* (c) Copyright by Varitronic AG 3177 Laupen Tel 031/747 60 60 Fax ..60 62 */
/*##########################################################################*/
/****************************************************************************/
/* File			: FM25xx.C                                       		    */
/* Kunde		: Varitronic AG                                     		*/
/* Projekt		: Diverse                                              		*/
/* Datum		: 31.01.2006                                           		*/
/* Autor        : A.Haeusler    	                                    	*/
/* Tools		: IBM PC, WIN-XP, IAR C-Compiler V1.53f             		*/
/*##########################################################################*/
/****************************************************************************/ 

/********************************** INCLUDES ********************************/

#include "Ioh82378.h"	// H8S2378 Header File 
#include <Inh8.h>   

#include "Def.h"
#include "FM25xx.h"     

/**************************** GLOBALE VARIABLEN *****************************/  


/****************************************************************************/
/******************************** FUNKTIONEN ********************************/
/****************************************************************************/

/*############################################################################
#                                                                            #
#       Funktions-Name  : init_spi_eeprom()                                  #
#                                                                            #
##############################################################################
#                                                                            #
#       Funktion        : Routine zur Initialisierung der SPI-Schnittstelle  #
#       Inputparameter  : ----- 			                                 #
#       Outputparameter : -----                          					 #
#       Autor           : AH                                                 #
#       Datum           : 31.01.2006                                         #
#       Revision        : 1.0                                                #
#                                                                            #
############################################################################*/
unsigned char init_spi_eeprom(void)
{
	
	SPI_EEPROM_WP_ON;	    // Schreibschutz ON
	
	CS1_OFF;    			// Setzt CS 1 aktiv
	CS2_OFF;    			// Setzt CS 2 aktiv
	CS3_OFF;    			// Setzt CS 3 aktiv
	CS4_OFF;    			// Setzt CS 4 aktiv

	SCK_1;     			    // Setzt Clockleitung Low      	  
	SO_1;			        // Setzt Dataleitung Low                                                                  			
		
	SET_SPI_PORT_DIR;		// Setzt der Portpins
	
}

/*############################################################################
#                                                                            #
#       Funktions-Name  : spi_eeprom_set_write_enable                        #
#                                                                            #
##############################################################################
#                                                                            #
#       Funktion        : Kommando > Set Write Enable Latch					 #
#       Inputparameter  : Chipselect, Kommando 			                     #
#       Outputparameter : -----                          					 #
#       Autor           : AH                                                 #
#       Datum           : 31.01.2006                                         #
#       Revision        : 1.0                                                #
#                                                                            #
############################################################################*/
void spi_eeprom_write_command(unsigned char chip_select, unsigned char command); 
{

// SPI Mode 3 ----------------------------------------------------------------

 unsigned char i=0;

	// Chip Select Signal freigeben
 	if(chip_select==1) CS1_ON;
 	if(chip_select==2) CS2_ON;
 	if(chip_select==3) CS3_ON;
 	if(chip_select==4) CS4_ON;

	SPI_TCSU; 	// Timing	

	SCK_0;		// Clock = 0;

	SPI_TCL;	// Timing

	
 	for (i=0;i<8;i++ )                              
 	{
  
  		if ((command & 0x80) == 0x80) SO_1;		// Setzt Dataleitung High
   		 else SO_0;								// Setzt Dataleitung Low
   
  		command=command << 1; 

		SPI_TSU;	// Timing

		SCK_1;		// Clock = 1

		SPI_TCH;	// Timing

		SCK_0;		// Clock = 0

		SPI_TCL;	// Timing		

 	}

	SPI_TCSH;		// Timing

	// Chip Select Signal freigeben
 	if(chip_select==1) CS1_OFF;
 	if(chip_select==2) CS2_OFF;
 	if(chip_select==3) CS3_OFF;
 	if(chip_select==4) CS4_OFF;

	SCK_1;		// Clock = 1

	SPI_TD;		// Timing

}

/*############################################################################
#                                                                            #
#       Funktions-Name  : spi_eeprom_write_status()                          #
#                                                                            #
##############################################################################
#                                                                            #
#       Funktion        : Write Status Register					             #
#       Inputparameter  : Status Byte		                                 #
#       Outputparameter : -----                          					 #
#       Autor           : AH                                                 #
#       Datum           : 31.01.2006                                         #
#       Revision        : 1.0                                                #
#                                                                            #
############################################################################*/
void spi_eeprom_write_status(unsigned char chip_select,unsigned char status_byte); 
{

// SPI Mode 3 ----------------------------------------------------------------

 unsigned char i=0;
 unsigned char command;

 	spi_eeprom_write_command(CMD_WREN);	// Write Enable senden

 	command=CMD_WRSR;					// Write Status Command setzen

	// Chip Select Signal freigeben
 	if(chip_select==1) CS1_ON;
 	if(chip_select==2) CS2_ON;
 	if(chip_select==3) CS3_ON;
 	if(chip_select==4) CS4_ON;

	SPI_TCSU; 	// Timing	

	SCK_0;		// Clock = 0;

	SPI_TCL;	// Timing

	// Send Command
 	for (i=0;i<8;i++ )                              
 	{
  
  		if((command & 0x80) == 0x80) SO_1;		// Setzt Dataleitung High
   		 else SO_0;								// Setzt Dataleitung Low
   
  		command=command << 1; 

		SPI_TSU;	// Timing

		SCK_1;		// Clock = 1

		SPI_TCH;	// Timing

		SCK_0;		// Clock = 0

		SPI_TCL;	// Timing		

 	}

	SPI_TSU;	// Timing

	// Send Status Byte
	for (i=0;i<8;i++ )                              
 	{
  
  		if ((status_byte & 0x80) == 0x80) SO_1;	// Setzt Dataleitung High
   		 else SO_0;								// Setzt Dataleitung Low
   
  		status_byte << 1; 

		SPI_TSU;	// Timing

		SCK_1;		// Clock = 1

		SPI_TCH;	// Timing

		SCK_0;		// Clock = 0

		SPI_TCL;	// Timing		

 	}


	SPI_TCSH;		// Timing

	// Chip Select Signal freigeben
 	if(chip_select==1) CS1_OFF;
 	if(chip_select==2) CS2_OFF;
 	if(chip_select==3) CS3_OFF;
 	if(chip_select==4) CS4_OFF;

	SCK_1;		// Clock = 1

	SPI_TD;		// Timing

}

/*############################################################################
#                                                                            #
#       Funktions-Name  : spi_eeprom_read_status()                           #
#                                                                            #
##############################################################################
#                                                                            #
#       Funktion        : Write Status Register					             #
#       Inputparameter  : Chip Select		                                 #
#       Outputparameter : Status Byte                          				 #
#       Autor           : AH                                                 #
#       Datum           : 31.01.2006                                         #
#       Revision        : 1.0                                                #
#                                                                            #
############################################################################*/
unsigned char spi_eeprom_read_status(unsigned char chip_select); 
{

// SPI Mode 3 ----------------------------------------------------------------

 unsigned char i=0;
 unsigned char command;
 unsigned char data;
 	

 	command=CMD_RDSR;					// Read Status Command setzen

	// Chip Select Signal freigeben
 	if(chip_select==1) CS1_ON;
 	if(chip_select==2) CS2_ON;
 	if(chip_select==3) CS3_ON;
 	if(chip_select==4) CS4_ON;

	SPI_TCSU; 	// Timing	

	SCK_0;		// Clock = 0;

	SPI_TCL;	// Timing

	// Send Command
 	for (i=0;i<8;i++ )                              
 	{
  
  		if ((command & 0x80) == 0x80) SO_1;		// Setzt Dataleitung High
   		 else SO_0;								// Setzt Dataleitung Low
   
  		command=command << 1; 

		SPI_TSU;	// Timing

		SCK_1;		// Clock = 1

		SPI_TCH;	// Timing

		SCK_0;		// Clock = 0

		SPI_TCL;	// Timing		

 	}

	SPI_TSU;	// Timing

	// Datenbyte einlesen
	data = 0;   

 	for(a=0;a<8;a++) 
  	{           		  

		data = data << 1;
		                         
    	if((SI_IN_1)==1) data |= 0x01;
		 else data &= ~0x01;  

		SPI_TH;		// Timing

		SCK_1;		// Clock = 1  

		SPI_TCH;	// Timing

		SCK_0;		// Clock = 0

		SPI_TCL;	// Timing	
		                                                                         
  	}   


	// Chip Select Signal freigeben
 	if(chip_select==1) CS1_OFF;
 	if(chip_select==2) CS2_OFF;
 	if(chip_select==3) CS3_OFF;
 	if(chip_select==4) CS4_OFF;

	SCK_1;		// Clock = 1

	SPI_TD;		// Timing

	return (data);               /* data return function */ 

}

/*############################################################################
#                                                                            #
#       Funktions-Name  : spi_write_byte()                          		 #
#                                                                            #
##############################################################################
#                                                                            #
#       Funktion        : Schreiben eines Bytes an Adresse n    			 #
#       Inputparameter  : Chipselect, Adresse, Datan                         #
#       Outputparameter : -----                          					 #
#       Autor           : AH                                                 #
#       Datum           : 31.01.2006                                         #
#       Revision        : 1.0                                                #
#                                                                            #
############################################################################*/
void spi_write_byte(unsigned char chip_select, 
					unsigned short adresse, 
					unsigned char daten); 
{

// SPI Mode 3 ----------------------------------------------------------------

 unsigned char i=0;
 unsigned char command;

 	spi_eeprom_write_command(CMD_WREN);	// Write Enable senden

 	command=CMD_WRITE;
	adresse &= 0x7FFF;

	// Chip Select Signal freigeben
 	if(chip_select==1) CS1_ON;
 	if(chip_select==2) CS2_ON;
 	if(chip_select==3) CS3_ON;
 	if(chip_select==4) CS4_ON;

	SPI_TCSU; 	// Timing	

	SCK_0;		// Clock = 0;

	SPI_TCL;	// Timing

	// Send Command
 	for (i=0;i<8;i++ )                              
 	{
  
  		if ((command & 0x80) == 0x80) SO_1;		// Setzt Dataleitung High
   		 else SO_0;								// Setzt Dataleitung Low
   
  		command=command << 1; 

		SPI_TSU;	// Timing

		SCK_1;		// Clock = 1

		SPI_TCH;	// Timing

		SCK_0;		// Clock = 0

		SPI_TCL;	// Timing		

 	}

	SPI_TSU;	// Timing

	// Send Adresse
	for (i=0;i<16;i++ )                              
 	{
  
  		if ((adresse & 0x80) == 0x80) SO_1;	// Setzt Dataleitung High
   		 else SO_0;								// Setzt Dataleitung Low
   
  		adresse << 1; 

		SPI_TSU;	// Timing

		SCK_1;		// Clock = 1

		SPI_TCH;	// Timing

		SCK_0;		// Clock = 0

		SPI_TCL;	// Timing		

 	}

	SPI_TSU;	// Timing

	// Send Byte
	for (i=0;i<8;i++ )                              
 	{
  
  		if ((daten & 0x80) == 0x80) SO_1;	// Setzt Dataleitung High
   		 else SO_0;								// Setzt Dataleitung Low
   
  		daten << 1; 

		SPI_TSU;	// Timing

		SCK_1;		// Clock = 1

		SPI_TCH;	// Timing

		SCK_0;		// Clock = 0

		SPI_TCL;	// Timing		

 	}


	SPI_TCSH;		// Timing

	// Chip Select Signal freigeben
 	if(chip_select==1) CS1_OFF;
 	if(chip_select==2) CS2_OFF;
 	if(chip_select==3) CS3_OFF;
 	if(chip_select==4) CS4_OFF;

	SCK_1;		// Clock = 1

	SPI_TD;		// Timing

}


habe mal mit meiner Schreibroutine des 25LC040 verglichen.
1. Chipselect Low
2. Write Enable Latch (bei mir 0x06)
3. Chipselect auf High und kurz warten (CS disable time)
4. Chipselect auf Low und raus mit dem Schreibbefehl, der Adresse und
den Daten

